@@ -52,6 +52,8 @@
 #include "log.h"
 #include "mainwindow.h"
 #include "statusbar.h"
+#include "msgcache.h"
+#include "timing.h"
 
 #include <gtk/gtk.h>
 #include <dirent.h>
@@ -102,6 +104,7 @@ static FolderItem *vcal_create_folder(Folder * folder,
 static gint vcal_create_tree(Folder *folder);
 static gint vcal_remove_folder(Folder *folder, FolderItem *item);
 static gboolean vcal_scan_required(Folder *folder, FolderItem *item);
+static void vcal_set_mtime(Folder *folder, FolderItem *item);
 static void vcal_change_flags(Folder *folder, FolderItem *_item, MsgInfo *msginfo, MsgPermFlags newflags);
 static void new_meeting_cb(FolderView *folderview, guint action, GtkWidget *widget);
 static void export_cal_cb(FolderView *folderview, guint action, GtkWidget *widget);
@@ -292,6 +295,7 @@ FolderClass *vcal_folder_get_class()
 		vcal_class.remove_folder = vcal_remove_folder;
 		vcal_class.rename_folder = vcal_rename_folder;
 		vcal_class.scan_required = vcal_scan_required;
+		vcal_class.set_mtime = vcal_set_mtime;
 		vcal_class.get_num_list = vcal_get_num_list;
 		vcal_class.set_batch = vcal_folder_set_batch;
 
@@ -329,7 +333,7 @@ static void vcal_folder_set_batch	(Folder		*folder,
 		/* ici */
 		item->batching = FALSE;
 		if (item->dirty)
-			vcal_folder_export();
+			vcal_folder_export(folder);
 		item->dirty = FALSE;
 	}
 }
@@ -474,8 +478,8 @@ static gint feed_fetch(FolderItem *fitem, MsgNumberList ** list, gboolean *old_u
 			orig_uid = g_strdup(uid);
 
 add_new:			
-			item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
-			item->evtlist = g_slist_append(item->evtlist, data);
+			item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
+			item->evtlist = g_slist_prepend(item->evtlist, data);
 			data = NULL;
 			debug_print("add %d : %s\n", num, uid);
 			g_free(uid);
@@ -485,35 +489,35 @@ add_new:
 				struct icaltimetype itt = icalproperty_get_dtstart(prop);
 				days = event_to_today(NULL, icaltime_as_timet(itt));
 				if (days == EVENT_PAST && past_msg == -1) {
-					item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
+					item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
 					data = icalfeeddata_new(NULL, EVENT_PAST_ID);
 					past_msg = num++;
 				} else if (days == EVENT_TODAY && today_msg == -1) {
-					item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
+					item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
 					data = icalfeeddata_new(NULL, EVENT_TODAY_ID);
 					today_msg = num++;
 				} else if (days == EVENT_TOMORROW && tomorrow_msg == -1) {
-					item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
+					item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
 					data = icalfeeddata_new(NULL, EVENT_TOMORROW_ID);
 					tomorrow_msg = num++;
 				} else if (days == EVENT_THISWEEK && thisweek_msg == -1) {
-					item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
+					item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
 					data = icalfeeddata_new(NULL, EVENT_THISWEEK_ID);
 					thisweek_msg = num++;
 				} else if (days == EVENT_LATER && later_msg == -1) {
-					item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
+					item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
 					data = icalfeeddata_new(NULL, EVENT_LATER_ID);
 					later_msg = num++;
 				}
 			} else {
 				if (past_msg == -1) {
-					item->numlist = g_slist_append(item->numlist, GINT_TO_POINTER(num));
+					item->numlist = g_slist_prepend(item->numlist, GINT_TO_POINTER(num));
 					data = icalfeeddata_new(NULL, EVENT_PAST_ID);
 					past_msg = num++;
 				}
 			}
 			if (data) {
-				item->evtlist = g_slist_append(item->evtlist, data);
+				item->evtlist = g_slist_prepend(item->evtlist, data);
 				data = NULL;
 			}
 			if (rprop) {
@@ -562,6 +566,9 @@ add_new:
 		evt = icalcomponent_get_next_component(
 			item->cal, type);
 	}
+	item->numlist = g_slist_reverse(item->numlist);
+	item->evtlist = g_slist_reverse(item->evtlist);
+	
 	*list = item->numlist ? g_slist_copy(item->numlist) : NULL;
 	debug_print("return %d\n", num);
 	return num;
@@ -573,31 +580,31 @@ add_new:
 	debug_print("should add %d %s? %d %d\n", n_msg, event->uid, status, status == ICAL_PARTSTAT_ACCEPTED); \
 	if (status == ICAL_PARTSTAT_ACCEPTED \
 	||  status == ICAL_PARTSTAT_TENTATIVE) { \
-		*list = g_slist_append(*list, GINT_TO_POINTER(n_msg)); \
+		*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
 		debug_print("add %d %s\n", n_msg, event->uid); \
 		n_msg++; \
 		days = event_to_today(event, 0); \
 	 \
 		if (days == EVENT_PAST && past_msg == -1) { \
-			*list = g_slist_append(*list, GINT_TO_POINTER(n_msg)); \
+			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
 			past_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, g_strdup_printf("%d", past_msg), g_strdup(EVENT_PAST_ID)); \
+			g_hash_table_insert(hash_uids, GINT_TO_POINTER(past_msg), g_strdup(EVENT_PAST_ID)); \
 		} else if (days == EVENT_TODAY && today_msg == -1) { \
-			*list = g_slist_append(*list, GINT_TO_POINTER(n_msg)); \
+			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
 			today_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, g_strdup_printf("%d", today_msg), g_strdup(EVENT_TODAY_ID)); \
+			g_hash_table_insert(hash_uids, GINT_TO_POINTER(today_msg), g_strdup(EVENT_TODAY_ID)); \
 		} else if (days == EVENT_TOMORROW && tomorrow_msg == -1) { \
-			*list = g_slist_append(*list, GINT_TO_POINTER(n_msg)); \
+			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
 			tomorrow_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, g_strdup_printf("%d", tomorrow_msg), g_strdup(EVENT_TOMORROW_ID)); \
+			g_hash_table_insert(hash_uids, GINT_TO_POINTER(tomorrow_msg), g_strdup(EVENT_TOMORROW_ID)); \
 		} else if (days == EVENT_THISWEEK && thisweek_msg == -1) { \
-			*list = g_slist_append(*list, GINT_TO_POINTER(n_msg)); \
+			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
 			thisweek_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, g_strdup_printf("%d", thisweek_msg), g_strdup(EVENT_THISWEEK_ID)); \
+			g_hash_table_insert(hash_uids, GINT_TO_POINTER(thisweek_msg), g_strdup(EVENT_THISWEEK_ID)); \
 		} else if (days == EVENT_LATER && later_msg == -1) { \
-			*list = g_slist_append(*list, GINT_TO_POINTER(n_msg)); \
+			*list = g_slist_prepend(*list, GINT_TO_POINTER(n_msg)); \
 			later_msg = n_msg++; \
-			g_hash_table_insert(hash_uids, g_strdup_printf("%d", later_msg), g_strdup(EVENT_LATER_ID)); \
+			g_hash_table_insert(hash_uids, GINT_TO_POINTER(later_msg), g_strdup(EVENT_LATER_ID)); \
 		} \
 	} \
 }
@@ -608,9 +615,10 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 	DIR *dp;
 	struct dirent *d;
 	int n_msg = 1;
-	gchar *snmsg = NULL;
 	gint past_msg = -1, today_msg = -1, tomorrow_msg = -1, 
 		thisweek_msg = -1, later_msg = -1;
+	START_TIMING("");
+	g_return_val_if_fail (*list == NULL, 0); /* we expect a NULL list */
 
 	debug_print(" num for %s\n", ((VCalFolderItem *)item)->uri);
 	
@@ -630,8 +638,8 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 	if (hash_uids != NULL)
 		g_hash_table_destroy(hash_uids);
 		
-	hash_uids = g_hash_table_new_full(g_str_hash, g_str_equal,
-					  g_free, g_free);
+	hash_uids = g_hash_table_new_full(g_direct_hash, g_direct_equal,
+					  NULL, g_free);
 	
 	while ((d = readdir(dp)) != NULL) {
 		VCalEvent *event = NULL;
@@ -641,8 +649,7 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 		||  !strcmp(d->d_name, "multisync")) 
 			continue;
 
-		snmsg = g_strdup_printf("%d",n_msg);
-		g_hash_table_insert(hash_uids, snmsg, g_strdup(d->d_name));
+		g_hash_table_insert(hash_uids, GINT_TO_POINTER(n_msg), g_strdup(d->d_name));
 		
 		event = vcal_manager_load_event(d->d_name);
 		if (!event)
@@ -659,7 +666,9 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 			enum icalparameter_partstat status = \
 				account ? vcal_manager_get_reply_for_attendee(event, account->address): ICAL_PARTSTAT_NEEDSACTION; \
 			VCAL_FOLDER_ADD_EVENT(event);
-			if (event->recur && strlen(event->recur)) {
+			if ((status == ICAL_PARTSTAT_ACCEPTED
+			     || status == ICAL_PARTSTAT_TENTATIVE) 
+			    && event->recur && *(event->recur)) {
         			struct icalrecurrencetype recur;
         			struct icaltimetype dtstart;
         			struct icaltimetype next;
@@ -668,7 +677,7 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 				struct icaldurationtype ical_dur;
 				int i = 0;
 
-				debug_print("dumping recurring events from main event\n");
+				debug_print("dumping recurring events from main event %s\n", d->d_name);
         			recur = icalrecurrencetype_from_string(event->recur);
 				dtstart = icaltime_from_string(event->dtstart);
 
@@ -699,8 +708,7 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 					vcal_manager_copy_attendees(event, nevent);
 					nevent->rec_occurence = TRUE;
 					vcal_manager_save_event(nevent, FALSE);
-					snmsg = g_strdup_printf("%d",n_msg);
-					g_hash_table_insert(hash_uids, snmsg, uid);
+					g_hash_table_insert(hash_uids, GINT_TO_POINTER(n_msg), uid);
 					VCAL_FOLDER_ADD_EVENT(nevent);
 					vcal_manager_free_event(nevent);
 					next = icalrecur_iterator_next(ritr);
@@ -716,7 +724,12 @@ static gint vcal_get_num_list(Folder *folder, FolderItem *item,
 
 	}
 	closedir(dp);
-	vcal_folder_export();
+	vcal_folder_export(folder);
+
+	vcal_set_mtime(folder, item);
+	
+	*list = g_slist_reverse(*list);
+	END_TIMING();
 	return g_slist_length(*list);
 }
 
@@ -741,19 +754,15 @@ static MsgInfo *vcal_get_msginfo(Folder * folder,
 {
 	MsgInfo *msginfo = NULL;
 	gchar *file;
-	gchar *snum = NULL;
 
 	debug_print("get_msginfo\n");
 	
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(num > 0, NULL);
 
-	snum = g_strdup_printf("%d", num);
-
 	file = vcal_fetch_msg(folder, item, num);
 
 	if (!file) {
-		g_free(snum);
 		return NULL;
 	}
 
@@ -765,13 +774,11 @@ static MsgInfo *vcal_get_msginfo(Folder * folder,
 
 		vcal_change_flags(NULL, NULL, msginfo, 0);
 
-		debug_print("  adding %s\n", snum);
+		debug_print("  adding %d\n", num);
 	}
 	unlink(file);
 	g_free(file);
 
-	g_free(snum);
-
 	debug_print("  got msginfo %p\n", msginfo);
 
 	return msginfo;
@@ -831,16 +838,18 @@ static gchar *vcal_fetch_msg(Folder * folder, FolderItem * item,
 				gint num)
 {
 	gchar *filename = NULL;
-	gchar *snum = NULL;
-	gchar *uid = NULL;
-	
+	const gchar *uid = NULL;
+
 	debug_print(" fetch for %s %d\n", ((VCalFolderItem *)item)->uri, num);
 	if (((VCalFolderItem *)item)->uri) 
 		return feed_fetch_item(item, num);
 
-	snum = g_strdup_printf("%d",num);
-	uid = g_hash_table_lookup(hash_uids, snum);
+	if (!uid) {
+		if (!hash_uids)
+			folder_item_scan_full(item, FALSE);
 
+		uid = g_hash_table_lookup(hash_uids, GINT_TO_POINTER(num));
+	}
 	if (uid && 
 	    (!strcmp(uid, EVENT_PAST_ID) ||
 	     !strcmp(uid, EVENT_TODAY_ID) ||
@@ -862,8 +871,6 @@ static gchar *vcal_fetch_msg(Folder * folder, FolderItem * item,
 
 		vcal_manager_free_event(event);
 	} 
-
-	g_free(snum);
 		
 	return filename;
 }
@@ -947,13 +954,48 @@ static gint vcal_remove_folder(Folder *folder, FolderItem *fitem)
 
 static gboolean vcal_scan_required(Folder *folder, FolderItem *item)
 {
-	return TRUE;
+	struct stat s;
+	VCalFolderItem *vitem = (VCalFolderItem *)item;
+
+	if (vitem->uri) {
+		return TRUE;
+	} else if (stat(vcal_manager_get_event_path(), &s) < 0) {
+		return TRUE;
+	} else if ((s.st_mtime > item->mtime) &&
+		(s.st_mtime - 3600 != item->mtime)) {
+		return TRUE;
+	}
+	return FALSE;
 }
 
 static gint vcal_folder_lock_count = 0;
 
-void vcal_folder_export(void)
+static void vcal_set_mtime(Folder *folder, FolderItem *item)
+{
+	struct stat s;
+	gchar *path = folder_item_get_path(item);
+
+	if (folder->inbox != item)
+		return;
+
+	g_return_if_fail(path != NULL);
+
+	if (stat(path, &s) < 0) {
+		FILE_OP_ERROR(path, "stat");
+		g_free(path);
+		return;
+	}
+
+	item->mtime = s.st_mtime;
+	debug_print("VCAL: forced mtime of %s to %ld\n", item->name?item->name:"(null)", item->mtime);
+	g_free(path);
+}
+
+void vcal_folder_export(Folder *folder)
 {	
+	FolderItem *item = folder?folder->inbox:NULL;
+	gboolean need_scan = folder?vcal_scan_required(folder, item):TRUE;
+
 	if (vcal_folder_lock_count) /* blocked */
 		return;
 	vcal_folder_lock_count++;
@@ -979,6 +1021,9 @@ void vcal_folder_export(void)
 				vcalprefs.export_freebusy_command, TRUE);
 	}
 	vcal_folder_lock_count--;
+	if (!need_scan && folder) {
+		vcal_set_mtime(folder, folder->inbox);
+	}
 }
 
 static void vcal_remove_event (Folder *folder, MsgInfo *msginfo)
@@ -1001,7 +1046,7 @@ static void vcal_remove_event (Folder *folder, MsgInfo *msginfo)
 	}
 	
 	if (!item || !item->batching)
-		vcal_folder_export();
+		vcal_folder_export(folder);
 	else if (item) {
 		item->dirty = TRUE;
 	}
@@ -1012,6 +1057,7 @@ static void vcal_change_flags(Folder *folder, FolderItem *_item, MsgInfo *msginf
 	EventTime date;
 
 	if (newflags & MSG_DELETED) {
+		MainWindow *mainwin = mainwindow_get_mainwindow();
 		/* delete the stuff */
 		msginfo->flags.perm_flags |= MSG_DELETED;
 		vcal_remove_event(folder, msginfo);
@@ -1128,7 +1174,7 @@ GSList * vcal_folder_get_waiting_events(void)
 
 			if (status == ICAL_PARTSTAT_ACCEPTED
 			||  status == ICAL_PARTSTAT_TENTATIVE) {
-				list = g_slist_append(list, event);
+				list = g_slist_prepend(list, event);
 			} else {
 				vcal_manager_free_event(event);
 			}
@@ -1139,7 +1185,7 @@ GSList * vcal_folder_get_waiting_events(void)
 
 	closedir(dp);
 		
-	return list;
+	return g_slist_reverse(list);
 }
 
 typedef struct _get_webcal_data {
@@ -1159,7 +1205,7 @@ static gboolean get_webcal_events_func(GNode *node, gpointer user_data)
 	for (cur = ((VCalFolderItem *)item)->evtlist; cur; cur = cur->next) {
 		IcalFeedData *fdata = (IcalFeedData *)cur->data;
 		if (fdata->event)
-			data->list = g_slist_append(data->list, fdata->event);
+			data->list = g_slist_prepend(data->list, fdata->event);
 	}
 	return FALSE;
 }
@@ -1175,7 +1221,7 @@ GSList * vcal_folder_get_webcal_events(void)
 	list = data->list;
 	g_free(data);
 
-	return list;
+	return g_slist_reverse(list);
 }
 
 gchar* get_item_event_list_for_date(FolderItem *item, EventTime date)
@@ -1214,7 +1260,7 @@ gchar* get_item_event_list_for_date(FolderItem *item, EventTime date)
 			} else
 				summary = g_strdup("-");
 
-			strs = g_slist_append(strs, summary);
+			strs = g_slist_prepend(strs, summary);
 		}
 	} else {
 		GSList *evtlist = vcal_folder_get_waiting_events();
@@ -1225,7 +1271,7 @@ gchar* get_item_event_list_for_date(FolderItem *item, EventTime date)
 			gchar *summary = NULL;
 			if (days == date) {
 				summary = g_strdup(event->summary);
-				strs = g_slist_append(strs, summary);
+				strs = g_slist_prepend(strs, summary);
 			}
 			vcal_manager_free_event(event);
 		}
@@ -1252,7 +1298,7 @@ gchar* get_item_event_list_for_date(FolderItem *item, EventTime date)
 	result = g_strdup_printf(_("\nThese are the events planned %s:\n"),
 			datestr);
 	
-	for (cur = strs; cur; cur = cur->next) {
+	for (cur = g_slist_reverse(strs); cur; cur = cur->next) {
 		int e_len = strlen(result);
 		int n_len = strlen((gchar *)cur->data);
 		if (e_len) {
@@ -1325,6 +1371,7 @@ void *url_read_thread(void *data)
 	curl_easy_setopt(curl_ctx, CURLOPT_WRITEFUNCTION, curl_recv);
 	curl_easy_setopt(curl_ctx, CURLOPT_WRITEDATA, &buffer);
 	curl_easy_setopt(curl_ctx, CURLOPT_TIMEOUT, prefs_common.io_timeout_secs);
+	curl_easy_setopt(curl_ctx, CURLOPT_NOSIGNAL, 1);
 #if LIBCURL_VERSION_NUM >= 0x070a00
 	curl_easy_setopt(curl_ctx, CURLOPT_SSL_VERIFYPEER, 0);
 	curl_easy_setopt(curl_ctx, CURLOPT_SSL_VERIFYHOST, 0);
@@ -1379,6 +1426,7 @@ gchar *vcal_curl_read(const char *url, gboolean verbose,
 	thread_data *td;
 #ifdef USE_PTHREAD
 	pthread_t pt;
+	pthread_attr_t pta;
 #endif
 	gchar *msg;
 	void *res;
@@ -1401,8 +1449,10 @@ gchar *vcal_curl_read(const char *url, gboolean verbose,
 	
 	g_free(msg);
 
-#if (defined USE_PTHREAD && ((defined __GLIBC__ && (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 3))) || !defined __GLIBC__))
-	if (pthread_create(&pt, PTHREAD_CREATE_JOINABLE, 
+#ifdef USE_PTHREAD
+	if (pthread_attr_init(&pta) != 0 ||
+	    pthread_attr_setdetachstate(&pta, PTHREAD_CREATE_JOINABLE) != 0 ||
+	    pthread_create(&pt, &pta, 
 			url_read_thread, td) != 0) {
 		url_read_thread(td);	
 	}
@@ -1549,10 +1599,21 @@ static void update_subscription_finish(const gchar *uri, gchar *feed, gboolean v
 
 	if (feed == NULL) {
 		if (verbose && manual_update) {
-			alertpanel_error(_("Could not retrieve the Webcal URL:\n%s\n%s"),
-					uri, error ? error:_("Unknown error"));
+			gchar *tmp; 
+			tmp = g_strdup(uri);
+			if (strlen(uri) > 61) {
+				tmp[55]='[';
+				tmp[56]='.';
+				tmp[57]='.';
+				tmp[58]='.';
+				tmp[59]=']';
+				tmp[60]='\0';
+			} 
+			alertpanel_error(_("Could not retrieve the Webcal URL:\n%s:\n\n%s"),
+					tmp, error ? error:_("Unknown error"));
+			g_free(tmp);
 		} else  {
-			log_error(LOG_PROTOCOL, _("Could not retrieve the Webcal URL:\n%s\n%s\n"),
+			log_error(LOG_PROTOCOL, _("Could not retrieve the Webcal URL:\n%s:\n\n%s\n"),
 					uri, error ? error:_("Unknown error"));
 		}
 		main_window_cursor_normal(mainwindow_get_mainwindow());
