@@ -26,7 +26,6 @@
 #include <sys/stat.h>
 #include "gettext.h"
 #include <ical.h>
-#include "utils.h"
 #include "vcalendar.h"
 #include "vcal_folder.h"
 #include "vcal_manager.h"
@@ -42,6 +41,7 @@
 #include <time.h>
 #include "folder.h"
 #include "quoted-printable.h"
+#include "utils.h"
 
 typedef struct _Answer Answer;
 
@@ -90,9 +90,10 @@ void vcal_manager_copy_attendees(VCalEvent *src, VCalEvent *dest)
 	while (cur && cur->data) {
 		Answer *a = (Answer *)cur->data;
 		Answer *b = answer_new(a->attendee, a->name, a->answer, a->cutype);
-		dest->answers = g_slist_append(dest->answers, b);
+		dest->answers = g_slist_prepend(dest->answers, b);
 		cur = cur->next;
 	}
+	dest->answers = g_slist_reverse(dest->answers);
 }
 
 gchar *vcal_manager_answer_get_text(enum icalparameter_partstat ans) 
@@ -180,9 +181,10 @@ GSList *vcal_manager_get_answers_emails(VCalEvent *event)
 	GSList *cur = event->answers;
 	while (cur && cur->data) {
 		Answer *b = (Answer *)cur->data;
-		new = g_slist_append(new, b->attendee);
+		new = g_slist_prepend(new, b->attendee);
 		cur = cur->next;
 	}
+	new = g_slist_reverse(new);
 	return new;	
 }
 
@@ -308,7 +310,7 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 	
 	subst_for_filename(sanitized_uid);
 
-	tmpfile = g_strdup_printf("%s%cevt-%d-%s", g_get_tmp_dir(),
+	tmpfile = g_strdup_printf("%s%cevt-%d-%s", get_tmp_dir(),
 				      G_DIR_SEPARATOR, getuid(), sanitized_uid);
 	g_free(sanitized_uid);
 
@@ -390,7 +392,7 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 		icalproperty_vanew_dtstart((icaltime_from_string(event->dtstart)), 0));
 	icalcomponent_add_property(ievent,
 		icalproperty_vanew_dtend((icaltime_from_string(event->dtend)), 0));
-	if (event->recur && strlen(event->recur)) {
+	if (event->recur && *(event->recur)) {
 		icalcomponent_add_property(ievent,
 			icalproperty_vanew_rrule((icalrecurrencetype_from_string(event->recur)), 0));
 	}
@@ -422,7 +424,7 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 
         icalcomponent_add_component(calendar, ievent);
 
-	if (event->url && strlen(event->url)) {
+	if (event->url && *(event->url)) {
 		attprop = icalproperty_new_url(event->url);
         	icalcomponent_add_property(ievent, attprop);
 	}
@@ -472,6 +474,7 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 		g_free(attendee);
 		return NULL;
 	}
+
 	headers = write_headers(account, event, is_pseudo_event, is_reply, is_pseudo_event);
 
 	if (!headers) {
@@ -486,7 +489,11 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 			       "\n"
 			       "%s", headers, icalcomponent_as_ical_string(calendar));
 	
-	str_write_to_file(body, tmpfile); 
+	if (str_write_to_file(body, tmpfile) < 0) {
+		g_free(tmpfile);
+		tmpfile = NULL;
+	}
+
 	chmod(tmpfile, S_IRUSR|S_IWUSR);
 
 	g_free(body);
@@ -495,6 +502,8 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 	g_free(attendee);
 	g_free(organizer);
 	
+	tzset();
+
 	return tmpfile;	
 }
 
@@ -576,7 +585,7 @@ gchar *vcal_manager_dateevent_dump(const gchar *uid, FolderItem *item)
 	
 	subst_for_filename(sanitized_uid);
 	
-	tmpfile = g_strdup_printf("%s%cevt-%d-%s", g_get_tmp_dir(),
+	tmpfile = g_strdup_printf("%s%cevt-%d-%s", get_tmp_dir(),
 				   G_DIR_SEPARATOR, getuid(), sanitized_uid);
 	g_free(sanitized_uid);
 
@@ -606,7 +615,10 @@ gchar *vcal_manager_dateevent_dump(const gchar *uid, FolderItem *item)
 			       "\n"
 			       "%s", headers, lines);
 	g_free(lines);
-	str_write_to_file(body, tmpfile); 
+	if (str_write_to_file(body, tmpfile) < 0) {
+		g_free(tmpfile);
+		tmpfile = NULL;
+	}
 	chmod(tmpfile, S_IRUSR|S_IWUSR);
 
 	g_free(body);
@@ -642,12 +654,12 @@ gchar *vcal_manager_icalevent_dump(icalcomponent *event, gchar *orga, icalcompon
 
 		subst_for_filename(sanitized_uid);
 
-		tmpfile = g_strdup_printf("%s%cevt-%d-%s", g_get_tmp_dir(),
+		tmpfile = g_strdup_printf("%s%cevt-%d-%s", get_tmp_dir(),
 					      G_DIR_SEPARATOR, getuid(), sanitized_uid);
 		g_free(sanitized_uid);
 		icalproperty_free(prop);
 	} else {
-		tmpfile = g_strdup_printf("%s%cevt-%d-%p", g_get_tmp_dir(),
+		tmpfile = g_strdup_printf("%s%cevt-%d-%p", get_tmp_dir(),
 				      G_DIR_SEPARATOR, getuid(), ievent);
 	}
 
@@ -720,7 +732,10 @@ gchar *vcal_manager_icalevent_dump(icalcomponent *event, gchar *orga, icalcompon
 			       "\n"
 			       "%s", headers, qpbody);
 	
-	str_write_to_file(body, tmpfile); 
+	if (str_write_to_file(body, tmpfile) < 0) {
+		g_free(tmpfile);
+		tmpfile = NULL;
+	}
 	chmod(tmpfile, S_IRUSR|S_IWUSR);
 
 	g_strfreev(lines);
@@ -752,14 +767,14 @@ VCalEvent * vcal_manager_new_event	(const gchar 	*uid,
 	event->organizer 	= g_strdup(organizer?organizer:"");
 	event->orgname	 	= g_strdup(orgname?orgname:"");
 
-	if (dtend && strlen(dtend)) {
+	if (dtend && *(dtend)) {
 		time_t tmp = icaltime_as_timet((icaltime_from_string(dtend)));
 		gchar buft[512];
 		tzset();
 		event->end	= g_strdup(ctime_r(&tmp, buft));
 	}
 	
-	if (dtstart && strlen(dtstart)) {
+	if (dtstart && *(dtstart)) {
 		time_t tmp = icaltime_as_timet((icaltime_from_string(dtstart)));
 		time_t tmp_utc = icaltime_as_timet((icaltime_from_string(dtstart)));
 		gchar buft[512];
@@ -838,9 +853,9 @@ PrefsAccount *vcal_manager_get_account_from_event(VCalEvent *event)
 	/* find an attendee corresponding to one of our accounts */
 	while (cur && cur->data) {
 		gchar *email = (gchar *)cur->data;
-		if (account_find_from_address(email)) {
+		if (account_find_from_address(email, FALSE)) {
 			g_slist_free(list);
-			return account_find_from_address(email);
+			return account_find_from_address(email, FALSE);
 		}
 		cur = cur->next;
 	}
@@ -937,7 +952,7 @@ void vcal_manager_save_event (VCalEvent *event, gboolean export_after)
 	}
  
  	if (export_after)
-		vcal_folder_export();
+		vcal_folder_export(NULL);
 }
 
 static VCalEvent *event_get_from_xml (const gchar *uid, GNode *node)
@@ -1040,11 +1055,12 @@ static VCalEvent *event_get_from_xml (const gchar *uid, GNode *node)
 				cutype = atoi(attr->value);
 		}
 
-		event->answers = g_slist_append(event->answers, answer_new(attendee, name, answer, cutype));
+		event->answers = g_slist_prepend(event->answers, answer_new(attendee, name, answer, cutype));
 		g_free(attendee);
 		g_free(name);
 		node = node->next;
 	}
+	event->answers = g_slist_reverse(event->answers);
 	
 	return event;
 }
